macroScript T3DTool

	category:"Undying"

	tooltip:"Undying T3D Import / Export Tool"

	icon:#("Undying", 1)

(

	---------------------------------

	-- Tests the plane equation (dot N p) against another point,

	-- p1 - it returns true if p1 is on the same plane, false if not

	fn samePlane N p p1 = (

		if ((dot N p) == (dot N p1)) do

			return true

		return false

	)

	

	---------------------------------

	fn sameNormals n o index = (

		normals = #()

		for i = 1 to o.numfaces where (i != index) do (

			if (getFaceNormal o i) == n do

				append normals i

		)

		return normals

	)

	

	---------------------------------

	-- takes an object, o, and looks for other faces in it that are

	-- on the same plane as face "index"

	fn samePlaneFaces o index = (

		spf = #()

		iVert = getVert o (getFace o index).x

		N = getFaceNormal o index

		for i = 1 to o.numfaces where (i != index) do (

			iFace = getFace o i

			iVert1 = getVert o iFace.x

			iVert2 = getVert o iFace.y

			iVert3 = getVert o iFace.z

			if 	(samePlane N iVert iVert1) and (samePlane N iVert iVert2) and (samePlane N iVert iVert3) do

				append spf i

		)

		append spf index -- append the source index

		return spf

	)

	

	-- new data structure for a tri object represented as a polyObject

	struct PolyObject (Vertices, Polygons, TVerts)

	

	-- gets the polylist and vert list of the object and returns a struct

	-- with the vertex list and polyList

	fn getPolyList o = (

		po = PolyObject()

		po.Polygons = #()

		po.Vertices = for i = 1 to o.numverts collect (getVert o i)

		tVertList = #()

		faceList = for k = 1 to o.numFaces collect false

		for k = 1 to o.numfaces do (

			if faceList[k] == false do (

				spf = samePlaneFaces o k

	

				if spf.count == 1 then (

					faceList[spf[1]] == true

					append po.polygons #(getFace o spf[1])

				) else (

					tmp = #()

					for k = 1 to spf.count do (

						faceList[spf[k]] = true

						append tmp (getFace o spf[k])

					)

					append po.polygons tmp

				)

			)

		)

		return po

	)

	

	

	fn sign n = (

		if n < 0 do return -1

		if n > 0 do return 1

		if n == 0 do return 0

	)

	

	fn sameSign v1 v2 = (

		if (sign v1.x) != (sign v2.x) do return false

		if (sign v1.y) != (sign v2.y) do return false

		if (sign v1.z) != (sign v2.z) do return false

		return true

	)

	-- is it a number or something that acts on a number?

	fn isNum str = (

		if str =="0" do return true

		if str =="1" do return true

		if str =="2" do return true

		if str =="3" do return true

		if str =="4" do return true

		if str =="5" do return true

		if str =="6" do return true

		if str =="7" do return true

		if str =="8" do return true

		if str =="9" do return true

		if str =="-" do return true

		if str =="+" do return true

		if str =="." do return true

		return false

	)

	

	fn getNum str = (

		if str.count > 0 then (

			if str[1] == "+" do

				str[1] = " "

			return (str as float)

		)

	)

	

	fn getXYZ str = (

		local n = #(), i

		num = ""

		for i = 1 to str.count do (

			if (isNum str[i]) then (

				num += str[i]

				if i == str.count do

					append n (getnum num)

			) else (

				if num != "" do (

					append n (getnum num)

					num = ""

				)

			)

		)

		return [n[1], -n[2], n[3]]

	)

	

	fn getValue1 str searchStr = (

		local i, j

		nm = ""

		index = findString str searchStr

		if (index != undefined) then (

			-- find the "="

			for i = index to str.count do

				if str[i] == "=" do

					for j = (i+1) to str.count do (

						c = str[j]

						if (c != " ") and (c != "\n") and (c != "\r") and (c != ",") and (c != ")") then (

							nm += str[j]

							if j == str.count do

								return nm

						) else

							return nm

					)

		) else (

			return undefined

		)

	)

	

	fn getRot str = (

		local inc = (360.0 / 65536)

		Pitch = getValue1 str "Pitch"

		Yaw = getValue1 str "Yaw"

		Roll = getValue1 str "Roll"

		if (Pitch == undefined) or (Pitch == OK) do Pitch = "0.0"

		if (Yaw == undefined) or (Yaw == OK) do Yaw = "0.0"

		if (Roll == undefined) or (Roll == OK) do Roll = "0.0"

		Pitch = inc * (mod ((getNum Pitch) as integer) 65536)

		Yaw = inc * (mod ((getNum Yaw) as integer) 65536)

		Roll = inc * (mod ((getNum Roll) as integer) 65536)		

		return (eulerAngles Roll -Pitch -Yaw)

	)

	

	-- takes an object and extracts it's rotation in terms on Unreal's rotations

	fn PYRRot o = (

		local inc = (65536.0 / 360.0)

		local r = o.transform as EulerAngles

		roll = (r.x * inc) as integer

		pitch = (r.y * inc) as integer

		yaw = (r.z * inc) as integer

		return #(Roll, -Pitch, -Yaw)

	)

	

	fn getLoc str = (

		x = getValue1 str "X="

		y = getValue1 str "Y="

		z = getValue1 str "Z="

		if (x == undefined) or (x == OK) do x = "0.0"

		if (y == undefined) or (y == OK) do y = "0.0"

		if (z == undefined) or (z == OK) do z = "0.0"

		try (x = getNum x) catch (x = 0.0)

		try (y = getNum y) catch (y = 0.0)

		try (z = getNum z) catch (z = 0.0)

		return [x,-y,z]

	)

	

	fn round num = (

		if num > 0 do return ((num+0.5) as integer)

		if num < 0 do return ((num-0.5) as integer)

		if num == 0 do return 0.0

	)

	

	fn ArbitraryRound num g = (

		num = num / (g as float)

		if num > 0 do

			num = ((num+0.5) as integer)

		if num < 0 do

			num = ((num-0.5) as integer)

	

		num = num * g

		return num

	)

	

	fn intPos p g sX sY sZ = (

		p1 = [0,0,0]

		if sX then p1.x = ArbitraryRound p.x g else p1.x = p.x

		if sY then p1.y = ArbitraryRound p.y g else p1.y = p.y

		if sZ then p1.z = ArbitraryRound p.z g else p1.z = p.z

		return p1

	)

	

	fn SnapSObject o granularity verts snapX snapY snapZ = (

		convertToMesh o

		for j = 1 to verts.count do (

			local iVert = getVert o verts[j]

			iVert = intPos iVert granularity snapX snapY snapZ

			setVert o verts[j] iVert

		)

		update o

	)

	

	

	fn SnapObject o granularity pOnly snapX snapY snapZ = (

		convertToMesh o

		o.pos = intPos o.pos granularity snapX snapY snapZ

		local oPos = o.pos

		o.pos = [0,0,0]

		if pOnly == false do (

			for j = 1 to o.numverts do (

				local iVert = getVert o j

				iVert = intPos iVert granularity snapX snapY snapZ

				setVert o j iVert

			)

			update o

		)

		o.pos = oPos

	)

	

	-- returns an Unreal texture gradient from 3 verts and their uv coordinates

	fn getTextureGrad pt0 pt1 pt2 val0 val1 val2 = (

	

		dpt1 = pt1 - pt0

		dpt2 = pt2 - pt0

	

		dv1 = val1 - val0

		dv2 = val2 - val0

	

		-- Compute the 2D matrix values, and invert the matrix.

		dpt11 = dot dpt1 dpt1

		dpt12 = dot dpt1 dpt2

		dpt22 = dot dpt2 dpt2

		factor = 1.0 / (dpt11 * dpt22 - dpt12 * dpt12)

	

		-- Compute the two gradients.

		g1 = (dv1 * dpt22 - dv2 * dpt12) * factor

		g2 = (dv2 * dpt11 - dv1 * dpt12) * factor

	

		p_grad_u = dpt1 * g1.x + dpt2 * g2.x

		p_grad_v = dpt1 * g1.y + dpt2 * g2.y

	

		-- Repeat process above, computing just one vector in the plane.

		dup1 = dot dpt1 p_grad_u

		dup2 = dot dpt2 p_grad_u

		dvp1 = dot dpt1 p_grad_v

		dvp2 = dot dpt2 p_grad_v

		

		fuctor = 1.0 / (dup1 * dvp2 - dvp1 * dup2)

		

		b1 = (val0.x * dvp2 - val0.y * dup2) * fuctor

		b2 = (val0.y * dup1 - val0.x * dvp1) * fuctor

	

		p_base = pt0 - (dpt1 * b1 + dpt2 * b2)

		

		p_grad_u *= 256

		p_grad_v *= 256

		

		if ((p_grad_u.x as string) == "-1.#IND") or ((p_grad_u.y as string) == "-1.#IND") or ((p_grad_u.z as string) == "-1.#IND") do p_grad_u = undefined

		if ((p_grad_v.x as string) == "-1.#IND") or ((p_grad_v.y as string) == "-1.#IND") or ((p_grad_v.z as string) == "-1.#IND") do p_grad_v = undefined

		if ((p_base.x as string) == "-1.#IND") or ((p_base.y as string) == "-1.#IND") or ((p_base.z as string) == "-1.#IND") do p_base = undefined

	

		return #(p_grad_u, p_grad_v, p_base)

	)

	---------------------------------------------------------------------

	---------------------------------------------------------------------

	---------------------------------------------------------------------

	

	rollout UndyingT3DTools "Undying T3D Tools"

	(

		local Unr,							-- rollout for the floater window

					window_w = 300,		-- floater window width

					window_h = 630,		-- floater window height

					debug = false,

					version = "0.96",

					date = "5/11/00"

	

		local CSGTypes = #("CSG_Add", "CSG_Subtract"),

					gValues = #(1,2,4,8,16,32,64,128,256,512),

					g = 1,

					c = (1.0/256)



		group "Map / Brush Import Options" (

			colorpicker AddCol "Brush Add Col:" color:(color 87 107 193) align:#right

			colorpicker SubCol "Brush Subtract Col:" color:(color 211 203 116) align:#right

			colorpicker movCol "Mover Col:" color:(color 199 4 255) align:#right

			checkbox importLights "Import Lights" offset:[0,-40]

			checkBox flipSubtracts "Flip Normals on Incoming Subtract Brushes" offset:[0,30]

			button importT3D "Import Unreal T3D File" align:#left width:250 height:35

			button analyzeT3D "Analyze T3D File" align:#left offset:[0,0]

			button analyzeT3DTextures "Analyze T3D File Textures" align:#right offset:[0,-26]

		)



		group "Map / Brush Export Options" (

			checkBox ExportAsOne "Export Selection as Single Brush"

			checkBox ExportPolyList "Export as Polygons" checked:false enabled:false

			checkBox SnapPositionsOnExport "Snap Positions on Export"

			checkbox PreserveStack "Preserve Modifier Stack" checked:true

			checkbox ExportFiles "Export as Individual Files"

			button exportT3D "Export Selection to T3D File" width:250 height:35

		)

		

		group "Export Special" (

			editText ClassName "Class Name: "

			editText Tag "Tag: "

			button ExportSpecial "Export Selection" align:#left

			button CreatePoints "Create Points" align:#right offset:[0,-26]

		)



		group "Snap Positions" (

			checkBox posOnly "Snap Object Position Only" align:#left

			DropDownList GranList items:#("1 Unit", "2 Units", "4 Units", "8 Units", "16 Units", "32 Units", "64 Units", "128 Units", "256 Units", "512 Units", "Other Units") width:80 selection:1 height:12

			spinner gran "Granularity:" range:[1,1024,1] type:#integer align:#right fieldWidth:45 offset:[0,-22] enabled:false

			checkbox restrict "Restrict Axes"

			checkbox rX "x" enabled:false offset:[110,-20]

			checkbox rY "y" enabled:false offset:[150,-20]

			checkbox rZ "z" enabled:false offset:[190,-20]

			button IntegerPos "Snap Positions" width:120 align:#left

			button snapSelection "Snap SO Selection" width:120 align:#right offset:[0,-26]

		)

		

		group "Unit Conversion" (

			spinner MetersSpinner "Meters:" range:[0,100000,0] type:#float offset:[0,0] fieldwidth:50 align:#left

			spinner cmSpinner "Centimeters" range:[0,100000,0] type:#float offset:[0,-20] fieldwidth:50 align:#right

			spinner FeetSpinner "Feet:" range:[0,100000,0] type:#float offset:[11,0] fieldwidth:50 align:#left

			spinner InchesSpinner "Inches:" range:[0,100000,0] type:#float offset:[0,-20] fieldwidth:50 align:#right

			spinner AeonsUnits "Undying Units:" range:[0,100000,0] type:#float offset:[0,0] fieldwidth:50

			label foo "" offset:[0,-20]

		)



		group "Status" (

			label status1

		)

	

-- end Interface stuff



-- New function defs



fn ExportObject o f = (

	local i

	

	-- light

	if classOf o == Omnilight do (

		format "Begin Actor Class=Light Name=%\n" o.name to:f

		format "  Tag=Light\n" to:f

		format "  Location=(X=%,Y=%,Z=%)\n" o.Pos.x (-o.Pos.y) o.Pos.z to:f

		format "  LightBrightness=%\n" ((o.multiplier * 64) as integer) to:f

		format "  LightHue=%\n" (o.color.h as integer) to:f

		format "  LightSaturation=%\n" ((abs(o.color.s - 255)) as integer) to:f

		format "  Name=%\n" o.name to:f

		format "End Actor\n" to:f

	)



	-- geometry

	if superClassOf o == GeometryClass do (



		-- snap positions on export

		if SnapPositionsOnExport.checked do

			if restrict.checked then

				SnapObject o g posOnly.checked rX.state rY.state rZ.state

			else

				SnapObject o g posOnly.checked true true true



		format "Begin Actor Class=Brush Name=%\n" o.name to:f

		convertToMesh o



		-- Get the PRS, and set the PRS to unnit and origin values

		oPos = o.pos

		oQuatRot = o.rotation

		oRot = o.rotation as eulerAngles

		-- oScale = o.scale

		oScale = [1,1,1]

		rot = PYRRot o



		o.rotation = quat 0 0 0 1

		-- o.scale = [1,1,1]

		o.pos = [0,0,0]



		-- object is mapped

		local hasTVerts = false

		if o.numtverts > 0 do hasTVerts = true



		-- get the channel of the gBuffer object channel

		iChannel = (o.gBufferChannel+1)

		if (iChannel != 1) and (iChannel != 2) do

			iChannel = 1

		format "  CsgOper=%\n" CSGTypes[iChannel] to:f

		format "  MainScale=(SheerAxis=SHEER_ZX)\n" to:f

		format "  PostScale=(Scale=(X=%,Y=%,Z=%),SheerAxis=SHEER_ZX)\n" oScale.x oScale.y oScale.z to:f

		format "  Tag=Brush\n" to:f

		format "  Location=(X=%,Y=%,Z=%)\n" oPos.x (-oPos.y) oPos.z to:f

		format "  Rotation=(Pitch=%,Yaw=%,Roll=%)\n" rot[2] rot[3] rot[1] to:f

		format "  Begin Brush Name=Model%\n" i to:f

		format "    Begin PolyList\n" to:f

		-- write out the polygon



		texName = #()

		if o.mat != undefined then (

			if classOf o.mat == Standardmaterial do (

				if classOf o.mat.diffuseMap == bitmapTexture do (

					if o.mat.diffusemap.filename != "" do (

						-- print (getFilenameFile o.mat.diffusemap.filename)

						append texName (getFilenameFile o.mat.diffusemap.filename)

					)

				)

			) -- std mat

			

			if classof o.mat == MultiMaterial do (

				ns = o.mat.numsubs

				for i = 1 to ns do (

					if classOf o.mat[i] == StandardMaterial do (

						if classOf o.mat[i].diffuseMap == bitmapTexture do (

							if o.mat[i].diffusemap.filename != "" do (

								-- print (getFilenameFile o.mat[i].diffusemap.filename)

								append texName (getFilenameFile o.mat[i].diffusemap.filename)

							)

						)

					)

				)

			) -- multi mat

		) else (

			-- print "Material not defined!"

		)



		for j = 1 to o.numfaces do (

			tName = undefined



			if texName.count > 1 then 

				tName = texName[(getFaceMatID o j)]

			else

				tName = texName[1]



			if (tName == undefined) then

				format "      Begin Polygon\n" to:f

			else

				format "      Begin Polygon Texture=%\n" tName to:f



			-- get the face and texture face

			iFace = getFace o j



			if hasTVerts do (

				iTVFace = getTVFace o j



				iTVert1 = getTVert o iTVFace.x

				iTVert2 = getTVert o iTVFace.y

				iTVert3 = getTVert o iTVFace.z



				-- convert to point2's

				iTVert1 = [iTVert1.x, iTVert1.y]

				iTVert2 = [iTVert2.x, iTVert2.y]

				iTVert3 = [iTVert3.x, iTVert3.y]

			)



			iVert1 = getVert o iFace.x

			iVert2 = getVert o iFace.y

			iVert3 = getVert o iFace.z



			if hasTVerts do (

				uvData = undefined

				uvData = getTextureGrad iVert1 iVert2 iVert3 iTVert1 iTVert2 iTVert3

				

				if uvData != undefined do (

					-- format "uvData: % %\n" j uvData

					if (findItem uvData undefined) == 0  do (

						format "        Origin\t%,%,%\n" uvData[3].x (-uvData[3].y) uvData[3].z to:f

						format "        TextureU\t%,%,%\n" uvData[1].x (-uvData[1].y) uvData[1].z to:f

						format "        TextureV\t%,%,%\n" uvData[2].x (-uvData[2].y) uvData[2].z to:f

					)

				)

			)

	

			format "        Vertex\t%,%,%\n" iVert2.x (-iVert2.y) iVert2.z to:f

			format "        Vertex\t%,%,%\n" iVert1.x (-iVert1.y) iVert1.z to:f

			format "        Vertex\t%,%,%\n" iVert3.x (-iVert3.y) iVert3.z to:f

			format "      End Polygon\n" to:f

		)

		format "    End PolyList\n" to:f

		format "  End Brush\n" to:f

		format "Brush=Model'MyLevel.Model%'\n" i to:f

		format "Name=%\n" o.name to:f

		format "End Actor\n\n" to:f



		-- set the object back to it's original PRS

		-- o.scale = oScale

		o.rotation = oQuatRot

		o.pos = oPos -- move it back



		-- delete the copy

	)

)





fn ExportPolyObject o f = (

	-- light

	if classOf o == Omnilight do (

		format "Begin Actor Class=Light Name=%\n" o.name to:f

		format "  Tag=Light\n" to:f

		format "  Location=(X=%,Y=%,Z=%)\n" o.Pos.x (-o.Pos.y) o.Pos.z to:f

		format "  LightBrightness=%\n" ((o.multiplier * 64) as integer) to:f

		format "  LightHue=%\n" (o.color.h as integer) to:f

		format "  LightSaturation=%\n" ((abs(o.color.s - 255)) as integer) to:f

		format "  Name=%\n" o.name to:f

		format "End Actor\n" to:f

	)



	-- geometry

	if superClassOf o == GeometryClass do (



		-- snap positions on export

		if SnapPositionsOnExport.checked do

			if restrict.checked then

				SnapObject o g posOnly.checked rX.state rY.state rZ.state

			else

				SnapObject o g posOnly.checked true true true



		format "Begin Actor Class=Brush Name=%\n" o.name to:f

		convertToMesh o



		-- Get the PRS, and set the PRS to unnit and origin values

		oPos = o.pos

		oQuatRot = o.rotation

		oRot = o.rotation as eulerAngles

		-- oScale = o.scale

		oScale = [1,1,1]

		rot = PYRRot o



		o.rotation = quat 0 0 0 1

		-- o.scale = [1,1,1]

		o.pos = [0,0,0]



		pl = getPolyList o



		-- object is mapped

		local hasTVerts = false

		if o.numtverts > 0 do hasTVerts = true



		-- get the channel of the gBuffer object channel

		iChannel = (o.gBufferChannel+1)

		if (iChannel != 1) and (iChannel != 2) do

			iChannel = 1

		format "  CsgOper=%\n" CSGTypes[iChannel] to:f

		format "  MainScale=(SheerAxis=SHEER_ZX)\n" to:f

		format "  PostScale=(Scale=(X=%,Y=%,Z=%),SheerAxis=SHEER_ZX)\n" oScale.x oScale.y oScale.z to:f

		format "  Tag=Brush\n" to:f

		format "  Location=(X=%,Y=%,Z=%)\n" oPos.x (-oPos.y) oPos.z to:f

		format "  Rotation=(Pitch=%,Yaw=%,Roll=%)\n" rot[2] rot[3] rot[1] to:f

		format "  Begin Brush Name=Model%\n" i to:f

		format "    Begin PolyList\n" to:f

		-- write out the polygon

		for j = 1 to pl.polygons.count do (

			format "      Begin Polygon\n" to:f



			-- get the face and texture face

			iFace = pl.polygons[j]



			if hasTVerts do (

				iTVFace = getTVFace o j



				iTVert1 = getTVert o iTVFace.x

				iTVert2 = getTVert o iTVFace.y

				iTVert3 = getTVert o iTVFace.z



				-- convert to point2's

				iTVert1 = [iTVert1.x, iTVert1.y]

				iTVert2 = [iTVert2.x, iTVert2.y]

				iTVert3 = [iTVert3.x, iTVert3.y]

			)



			iVert1 = getVert o iFace.x

			iVert2 = getVert o iFace.y

			iVert3 = getVert o iFace.z





			if hasTVerts do (

				uvData = undefined

				uvData = getTextureGrad iVert1 iVert2 iVert3 iTVert1 iTVert2 iTVert3

				

				if uvData != undefined do (

					format "        Origin\t%,%,%\n" uvData[3].x (-uvData[3].y) uvData[3].z to:f

					format "        TextureU\t%,%,%\n" uvData[1].x (-uvData[1].y) uvData[1].z to:f

					format "        TextureV\t%,%,%\n" uvData[2].x (-uvData[2].y) uvData[2].z to:f

				)

			)

	

			format "        Vertex\t%,%,%\n" iVert2.x (-iVert2.y) iVert2.z to:f

			format "        Vertex\t%,%,%\n" iVert1.x (-iVert1.y) iVert1.z to:f

			format "        Vertex\t%,%,%\n" iVert3.x (-iVert3.y) iVert3.z to:f

			format "      End Polygon\n" to:f

		)

		format "    End PolyList\n" to:f

		format "  End Brush\n" to:f

		format "Brush=Model'MyLevel.Model%'\n" i to:f

		format "Name=%\n" o.name to:f

		format "End Actor\n\n" to:f



		-- set the object back to it's original PRS

		-- o.scale = oScale

		o.rotation = oQuatRot

		o.pos = oPos -- move it back



		-- delete the copy

	)

)







--------------------------------------------------------------------------

		on ExportFiles changed state do (

			if state then

				ExportAsOne.enabled = false

			else

				ExportAsOne.enabled = true

		)

		on restrict changed state do

			rX.enabled = rY.enabled = rZ.enabled = state



		-- Unit Conversion crap

		on MetersSpinner changed state do (

			FeetSpinner.value = MetersSpinner.value * 3.28100

			InchesSpinner.value = MetersSpinner.value * 39.3700

			AeonsUnits.value = FeetSpinner.value * 21.33333

			cmspinner.value = MetersSpinner.value * 100.00

		)

		

		on FeetSpinner changed state do (

			MetersSpinner.value = FeetSpinner.value * 0.304800

			InchesSpinner.value = FeetSpinner.value / 0.083300

			AeonsUnits.value = FeetSpinner.value * 21.33333

			cmspinner.value = MetersSpinner.value * 100.00

		)

		

		on InchesSpinner changed state do (

			MetersSpinner.value = InchesSpinner.value * 0.025400

			FeetSpinner.value = InchesSpinner.value * 0.083300

			AeonsUnits.value = FeetSpinner.value * 21.33333

			cmspinner.value = MetersSpinner.value * 100.00

		)

	

		on cmSpinner changed state do (

			MetersSpinner.value = cmSpinner.value * 0.01

			InchesSpinner.value = MetersSpinner.value * 39.3700

			FeetSpinner.value = InchesSpinner.value * 0.083300

			AeonsUnits.value = FeetSpinner.value * 21.33333

		)

	

		on AeonsUnits changed state do (

			InchesSpinner.value = AeonsUnits.value * 0.5625

			FeetSpinner.value = AeonsUnits.value * 0.046875

			MetersSpinner.value = InchesSpinner.value * 0.025400

			cmspinner.value = MetersSpinner.value * 100.00

		)



		-- End Unit Conversions

		

		on GranList selected s do (

			if s == 11 then (

				gran.enabled = true

				g = gran.value

			) else (

				gran.enabled = false

				g = gValues[s]

			)

		)



		on gran changed state do g = gran.value



		-- snapping objects / subobjects

		on snapSelection pressed do (

			sel = selection as array

			sc = sel.count

			for i = 1 to sc do (

				o = sel[i]

				if (superClassOf o == GeometryClass) do (

					s = getVertSelection o

					if restrict.checked then

						SnapSObject o g s rX.state rY.state rZ.state

					else

						SnapSObject o g s true true true

				)

			)

		)



		on IntegerPos pressed do (

			sel = selection as array

			sc = sel.count

			for i = 1 to sc do (

				o = sel[i]

				if (superClassOf o == GeometryClass) do

					if restrict.checked then

						SnapObject o g posOnly.checked rX.state rY.state rZ.state

					else

						SnapObject o g posOnly.checked true true true

			)

			select sel

		)

		

		on CreatePoints pressed do (

			sel = selection as array

			sc = sel.count 

			if (sc > 0) do (

				for i = 1 to sc do (

					o = sel[i]

					if (SuperClassOf o) == GeometryClass do (

						for j = 1 to o.numverts do (

							Point pos:(getVert o j)

						)

					)

				)

			)

		)

		

		-------------------------

		-- Export Special

		-------------------------

		on ExportSpecial pressed do (

			if (fName = getSaveFileName caption:"Specify Unreal File" Types:"Unreal File (*.t3d)|*.t3d|ALL (*.*)|*.*|") != undefined do (

				f = CreateFile fName

				sel = selection as array

				sc = sel.count

				if (sc > 0) do (

					format "Begin Map\n" to:f

					for i = 1 to sc do (

						o = sel[i]

						format "Begin Actor Class=%\n" ClassName.text o.name to:f

						format "  Tag=%\n" Tag.text to:f

						format "  Location=(X=%,Y=%,Z=%)\n" o.Pos.x (-o.Pos.y) o.Pos.z to:f

						format "  Name=%\n" o.name to:f

						format "End Actor\n" to:f

					)

					format "End Map\n" to:f

				)

				close f

			)

		)



		-------------------------

		-- Exporter

		-------------------------

		on ExportT3D pressed do (

			if (fName = getSaveFileName caption:"Specify Unreal File" Types:"Unreal File (*.t3d)|*.t3d|ALL (*.*)|*.*|") != undefined do (

				if ( ExportFiles.checked ) then (

					sel = selection as array

					sc = sel.count

					if sc > 0 do (

						for i = 1 to sc do (

							obj = sel[i]

							pth = getFileNamePath fName

							prefix = getFilenameFile fName

							f = createFile (pth + prefix + "_" + obj.name + ".t3d")

							format "; Unreal text format exported %\n" localTime to:f

							format "; DreamWorks Interactive\n" to:f

							format ";\n" to:f

							format "Begin Map\n" to:f

							-- If we export the geometry as one object, we'll read all the geometry data

							-- into some new structures and export from there.

							m = undefined

							o = undefined

							cnt = undefined



							status1.text = ("Exporting \"" + obj.name + "\"")



							-- we copy the object to preserve the modifer stack

							if PreserveStack.checked then

								m = copy obj

							else

								o = obj					



							-- Geometry

							if m != undefined then (

								ExportObject m f

								delete m

							) else (

								ExportObject o f

							)

							format "End Map\n" to:f

							close f

						) -- end loop

						status1.text = "Complete"

					)



				) else (



					f = createFile fName

					sel = selection as array

					sc = sel.count

					format "; Unreal text format exported %\n" localTime to:f

					format "; DreamWorks Interactive\n" to:f

					format ";\n" to:f

					format "Begin Map\n" to:f

					-- If we export the geometry as one object, we'll read all the geometry data

					-- into some new structures and export from there.

					m = undefined

					if ExportAsOne.checked do (

						-- this is the center of the 'new' object

						oneCenter = [0,0,0]

						cnt = 0

						for i = 1 to sc do

							if superclassOf sel[i] == GeometryClass do (

								cnt += 1

								oneCenter += sel[i].pos

							)

	

						oneCenter = oneCenter / cnt

	

						-- collect all the geometry data

						m = mesh vertices:#() faces:#()

						m.pivot = oneCenter

						for i = 1 to sc do (

							o = sel[i]

							if superClassOf o == GeometryClass do (

								attach m (copy o)

							)

						)	

					)

	 

					o = undefined

					cnt = undefined

					for i = 1 to sc do (

						status1.text = ("Exporting \"" + sel[i].name + "\"")

	

						-- we copy the object to preserve the modifer stack

						if PreserveStack.checked then

							o = copy sel[i]

						else

							o = sel[i]

	

						-- Geometry

						if (ExportAsOne.checked == false) then (

							if ExportPolyList.checked then (

								ExportPolyObject m f

							) else (

								ExportObject o f

							)

							

						) else (

							if classOf o == OmniLight do ExportObject o f

							if m != undefined do (

								if ExportPolyList.checked then (

									ExportPolyObject m f

								) else (

									ExportObject m f

									delete m

									m = undefined

								)

							)

						)

						if PreserveStack.checked do delete o

					)

					format "End Map\n" to:f

					close f

					status1.text = "Complete"

				)

			)

		)



		-------------------------

		-------------------------

		-- Importer

		-------------------------

		-------------------------

		on importT3D pressed do (

			if (fName = getOpenFileName caption:"Specify Unreal File" Types:"Unreal File (*.t3d)|*.t3d|ALL (*.*)|*.*|") != undefined do (

				addColor = AddCol.color

				subColor = SubCol.color

				f = openFile fName

				file = #()

				-- read the file in (not the comments, however)

				while not eof f do (

					CLine = readline f

					if CLine[1] != ";" do (

						append file Cline

					)

				)

				close f

				gc()

				-- start stepping throught the file

				fc = file.count  -- num lines

				Actors = #()

				-- find out how many actors we are loading

				for i = 1 to fc do

					if (findString file[i] "Begin Actor" != undefined) or (findString file[i] "End Actor" != undefined) do

						append Actors i

	

				inc = (1.0/Actors.count) * 100

				progressStart "Loading T3D File..."

				for i = 1 to Actors.count do (

					progressUpdate (inc * i) -- update the progressbar

					if (mod i 2) != 0 do (

						-- get the actor class

						-- BlockAll

	

						ActorClass = getValue1 file[Actors[i]] "Class"

						-- Actor is a Light

						if importLights.checked do

							if ActorClass == "Light" do (

		

								lightName = getValue1 file[Actors[i]] "Name"

								status1.text = ("Importing " + "\"" + lightName + "\"")

								ActorLocation = [0,0,0]

								radius = undefined

		

								-- default light color		

								lCol = white

								mult = 1.0

								for j = Actors[i] to Actors[i+1] do (

		

									-- Location

									if (findString file[j] "Location" != undefined) do (

										ActorLocation = getLoc file[j]

									)

									-- Brightness / Multiplier

									if (findString file[j] "LightBrightness" != undefined) do

										mult = ((getNum (getValue1 file[j] "LightBrightness")) as float) / 64.0

		

									-- Hue

									if (findString file[j] "LightHue" != undefined) do

										lCol.h = getNum (getValue1 file[j] "LightHue")

		

									-- Saturation

									if (findString file[j] "LightSaturation" != undefined) do

										lCol.s = abs (255 - (getNum (getValue1 file[j] "LightSaturation")))

		

									-- Light radius

									if (findString file[j] "LightRadius" != undefined) do

										radius = (getNum (getValue1 file[j] "LightRadius"))

										

								)

								-- Create it

								o = omniLight color:lCol pos:ActorLocation name:lightName

								o.multiplier = mult

								o.useFarAtten = o.useNearAtten = true

		

								if radius == undefined do radius = 64

		

								o.nearAttenStart = o.nearAttenEnd = o.farAttenStart = radius

								o.farAttenEnd = radius * 8.0

							)

	

						-- Actor is geometry

						if (ActorClass == "Brush") or (ActorClass == "Mover") do (

							brushName = getValue1 file[Actors[i]] "Name"

							status1.text = ("Importing " + "\"" + brushName + "\"")

							verts = #() -- verts

							f = #() -- faces

							TextureVertices = #()

							TextureFaces = #()

							Polygons = #()

							CSGType = "none"

							ActorLocation = [0,0,0]

							Rotation = quat 0 0 0 1

							objectScale = [1,1,1]

							for j = Actors[i] to Actors[i+1] do (

								-- Poly def

								if (findString file[j] "Begin Polygon" != undefined) or (findString file[j] "End Polygon" != undefined) do

									append Polygons j

	

								-- Location

								if (findString file[j] "Location" != undefined) and (findString file[j] "OldLocation" == undefined) do (

									ActorLocation = getLoc file[j]

								)

								-- Rotation

								if findString file[j] "Rotation" != undefined do

									Rotation = getRot file[j]



								-- Scale

								if findString file[j] "PostScale" != undefined do (

									objectScale = getLoc file[j]

									-- a value of 0 is interpreted as no scale, and thus a factor of 1

									if objectScale.x == 0 do objectScale.x = 1

									if objectScale.y == 0 do objectScale.y = 1

									if objectScale.z == 0 do objectScale.z = 1

								)

	

								-- CSG Type

								if findString file[j] "CsgOper" != undefined do

									CSGType = getValue1 file[j] "CsgOper"

							)

	

							uGrad = vGrad = undefined

							for j = 1 to Polygons.count do (

								if (mod j 2) != 0 do (

									vc = 0

									origin = tempNormal = newVert = undefined

									iVerts = #()

									for k = Polygons[j] to Polygons[j+1] do (



										if (findString file[k] "Texture=") != undefined then

											TMap = getValue1 file[k] "Texture"

										else

											TMap = undefined

										

										if findString file[k] "Origin" != undefined do

											origin = getXYZ file[k]



										if findString file[k] "Normal" != undefined do

											tempNormal = getXYZ file[k]

	

										-- uv's

										uGrad = vGrad = undefined

										if findString file[k] "TextureU" != undefined do

											uGrad = getXYZ file[k]



										if findString file[k] "TextureV" != undefined do

											vGrad = getXYZ file[k]



										-- vertices

										if findString file[k] "Vertex" != undefined do (

											vc += 1

											newVert = getXYZ file[k]

											append verts newVert

											append iVerts newVert

											-- tvs

											if (uGrad != undefined) and (vGrad != undefined) do (

												u = dot (newVert - origin) uGrad

												v = dot (newVert - origin) vGrad

												-- / by 256

												u = u * c

												v = v * c

												append textureVertices [u,v,1.0]

											)

										)

									)

	

									offset = (verts.count-vc)



									if (tempNormal != undefined) and (iVerts.count >= 3) do (

										iNormal = normalize (cross (iVerts[1] - iVerts[2]) (iVerts[1] - iVerts[3]))

										if (CSGType != "CSG_Subtract") then (

											for k = 1 to (vc-1) do (

												append f [offset+k,offset+1,offset+k+1]

												append TextureFaces [offset+k,offset+1,offset+k+1]

											)

										) else (

											if flipSubtracts.checked then (

												for k = 1 to (vc-1) do (

													append f [offset+1,offset+k,offset+k+1]

													append TextureFaces [offset+1,offset+k,offset+k+1]

												)

											) else (

												for k = 1 to (vc-1) do (

													append f [offset+k,offset+1,offset+k+1]

													append TextureFaces [offset+k,offset+1,offset+k+1]

												)

											)

										)

									)

								)

							)

							m = mesh vertices:verts faces:f tverts:textureVertices -- build the mesh

							setnumtverts m textureVertices.count



							for k = 1 to textureVertices.count do

								setTVert m k textureVertices[k]



							m.name = brushName

							m.transform = rotation as matrix3

							m.scale = objectScale

							m.pos = ActorLocation

							-- uv's

							if m.numtverts > 0 do (

								buildTVFaces m

								for k = 1 to f.count do

									setTVFace m k TextureFaces[k]

							)

							

							-- determine the wirecolor

							if ActorClass == "Mover" then

								m.wirecolor = movCol.color

							else (

								if CSGType == "CSG_Add" do

								(

									m.gbufferChannel = 0

									m.wirecolor = AddCol.color

								)

								if CSGType == "CSG_Subtract" do

								(

									m.gbufferChannel = 1

									m.wirecolor = SubCol.color

								)

								if CSGType == "none" do m.wirecolor = black

							)

						)

					)

				)

				ProgressEnd()

				gc()

				status1.text = "Complete"

			)

		) -- end import



		on analyzeT3D pressed do (

			if (fName = getOpenFileName caption:"Specify Unreal File" Types:"Unreal File (*.t3d)|*.t3d|ALL (*.*)|*.*|") != undefined do (

				addColor = AddCol.color

				subColor = SubCol.color

				f = openFile fName

				file = #()

				actorTypes = #()

				actorCounts = #()

				-- read the file in (not the comments, however)

				while not eof f do (

					CLine = readline f

					if CLine[1] != ";" do (

						append file Cline

					)

				)

				close f

				gc()

				-- start stepping throught the file

				fc = file.count  -- num lines

				Actors = #()

				-- find out how many actors we are loading

				for i = 1 to fc do

					if (findString file[i] "Begin Actor" != undefined) or (findString file[i] "End Actor" != undefined) do

						append Actors i

	

				inc = (1.0/Actors.count) * 100

				progressStart "Loading T3D File..."

				for i = 1 to Actors.count do (

					progressUpdate (inc * i) -- update the progressbar

					if (mod i 2) != 0 do (

						-- get the actor class

						-- BlockAll

	

						ActorClass = getValue1 file[Actors[i]] "Class"



						if (Idx = (findItem ActorTypes ActorClass)) == 0 then (

							append ActorTypes ActorClass

							append actorCounts 1

						) else (

							actorCounts[Idx] += 1

						)

					)

				)

				clearListener()

				for i = 1 to ActorTypes.count do (

					format "% : %\n" ActorTypes[i] ActorCounts[i]

				)



				ProgressEnd()

				gc()

				status1.text = "Complete"

			)

		) -- end import



		on analyzeT3DTextures pressed do (

			if (fName = getOpenFileName caption:"Specify Unreal File" Types:"Unreal File (*.t3d)|*.t3d|ALL (*.*)|*.*|") != undefined do (

				addColor = AddCol.color

				subColor = SubCol.color

				f = openFile fName

				file = #()

				actorTypes = #()

				actorCounts = #()

				-- read the file in (not the comments, however)

				while not eof f do (

					CLine = readline f

					if CLine[1] != ";" do (

						append file Cline

					)

				)

				close f

				gc()

				-- start stepping throught the file

				fc = file.count  -- num lines

				Textures = #()

				TextureNames = #()

				-- find out how many actors we are loading

				for i = 1 to fc do

					if (findString file[i] "Texture=" != undefined) do

						append Textures file[i]

	

				inc = (1.0/Textures.count) * 100

				progressStart "Loading T3D File..."

				for i = 1 to Textures.count do (

					progressUpdate (inc * i) -- update the progressbar

					if (idx = findString Textures[i] "Texture=") != undefined do (

						str = substring Textures[i] (idx+8) Textures[i].count

						for j = 1 to str.count do (

							if str[j] == " " do (

								s = substring str 1 (j-1)

								if (findItem TextureNames s == 0) do

									append TextureNames s

								exit;

							)

						)	

					)

				)



				ClearListener()

				for i = 1 to TextureNames.count do

					format "%\n" TextureNames[i]



				ProgressEnd()

				gc()

				status1.text = "Complete"

			)

		) -- end import



		

	)

	

	flt = NewRolloutFloater "Undying T3D Tool" 300 780

	addRollout UndyingT3DTools flt

)
